#!/usr/bin/env python
#

from gimpfu import *

############################################################################
# include, then use: erro_msg / info_msg
#---------------------------------------------------------------------------
def erro_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(0) # box
    gimp.message ("ERROR: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###
def info_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2) # error console
    gimp.message ("INFO: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###
############################################################################
#

def Only2hues (inImage, inDrawable) :
    
    pdb.gimp_context_push()
    # Indicates that the process has started.
    gimp.progress_init("Finding 2 prevailing hues in " + inDrawable.name + "...")

    # Set up an undo group, so the operation will be undone in one step.
    pdb.gimp_image_undo_group_start(inImage)

    # Get the layer position.
    pos = 0;
    for i in range(len(inImage.layers)):
        if(inImage.layers[i] == inDrawable):
            pos = i
    
    # Create a new layer to save the results.
    newLayer = gimp.Layer(inImage, inDrawable.name + " temp", inDrawable.width, inDrawable.height,
                          inDrawable.type, inDrawable.opacity, inDrawable.mode)
    inImage.add_layer(newLayer, pos)
    layerName = inDrawable.name
    if '.' in layerName:
        split = layerName.split(".", 1)
        layerName = split[0]
    
    # Clear the new layer.
    pdb.gimp_edit_clear(newLayer)
    newLayer.flush()
    
     
    try:
        # Calculate the number of tiles.
        tn = int(inDrawable.width / 128) # fix by Tin
        if(inDrawable.width % 128 > 0):
            tn += 1
        tm = int(inDrawable.height / 128) # fix by Tin
        if(inDrawable.height % 128 > 0):
            tm += 1
        
        # Iterate over the tiles.
        # - Find the most present hue
        
        TRed = 1;  TGreen = 1;  TBlue = 1;  TYellow = 1;  TCyan = 1;  TMagenta = 1;  TWhite = 1 # avoid zeroes
        TLuma = 0
        TPixels = 0
        # Totals.
        SumWHITEred = 0;  SumWHITEgreen = 0;  SumWHITEblue = 0
        SumREDred = 0;    SumREDgreen = 0;    SumREDblue = 0
        SumYELLOWred = 0; SumYELLOWgreen = 0; SumYELLOWblue = 0
        SumGREENred = 0;  SumGREENgreen = 0;  SumGREENblue = 0
        SumCYANred = 0;   SumCYANgreen = 0;   SumCYANblue = 0
        SumBLUEred = 0;   SumBLUEgreen = 0;   SumBLUEblue = 0
        SumMAGENTAred = 0;SumMAGENTAgreen = 0;SumMAGENTAblue = 0
        for i in range(tn):
            for j in range(tm):
                # Update the progress bar.
                gimp.progress_update(float(i*tm + j) / float(tn*tm))
                 
                # Get the tiles.
                srcTile = inDrawable.get_tile(False, j, i)
                dstTile = newLayer.get_tile(False, j, i)
                         
                
                # Iterate over the pixels of each tile.
                for x in range(srcTile.ewidth):
                    for y in range(srcTile.eheight):
                        # Get the pixel and calculate his gray value.
                        pixel = srcTile[x,y]
                        red = ord(pixel[0])
                        green = ord(pixel[1])
                        blue = ord(pixel[2])
                        gray = (ord(pixel[0]) + ord(pixel[1]) + ord(pixel[2]))/3
                        TPixels+=1
                        # Luminance (perceived option 1): (0.299*R + 0.587*G + 0.114*B)
                        Luma = (0.299*red + 0.587*green + 0.114*blue)
                        TLuma+=Luma
                        
                        Red=red/32
                        Green=green/32
                        Blue=blue/32
                         
                        if (Red==Green) and (Green==blue):               #white
                            TWhite+=1
                            SumWHITEred+=red;   SumWHITEgreen+=green;   SumWHITEblue+=blue
                        else:    
                        # count the hue
                          if (Red > Green) and (Red > Blue):             #Red
                            TRed+=1
                            SumREDred+=red;   SumREDgreen+=green;   SumREDblue+=blue
                          elif (Red == Green) and (Red > Blue):          #yellow
                            TYellow+=1
                            SumYELLOWred+=red;   SumYELLOWgreen+=green;   SumYELLOWblue+=blue
                          elif (Red < Green) and (Blue < Green):         #Green
                            TGreen+=1
                            SumGREENred+=red;   SumGREENgreen+=green;   SumGREENblue+=blue
                          elif (Red < Green) and (Blue == Green):        #cyan
                            TCyan+=1
                            SumCYANred+=red;   SumCYANgreen+=green;   SumCYANblue+=blue
                          elif (Red < Green) and (Green < Blue):         #Blue
                            TBlue+=1
                            SumBLUEred+=red;   SumBLUEgreen+=green;   SumBLUEblue+=blue
                          elif (Red > Green) and (Blue > Green):         #magenta
                            TMagenta+=1
                            SumMAGENTAred+=red;   SumMAGENTAgreen+=green;   SumMAGENTAblue+=blue
                            
        SortedList = sorted([TWhite,TRed,TYellow,TGreen,TCyan,TBlue,TMagenta])
        Hue1 = SortedList[6]
        if Hue1 == TWhite:
            Hue1name = "White"
            Hue1R = SumWHITEred/TWhite;  Hue1G = SumWHITEgreen/TWhite;  Hue1B = SumWHITEblue/TWhite;
            Hue2 = TWhite                                                                               # complementary colour
            Hue2name = "White"
        if Hue1 == TRed:
            Hue1name = "Red"
            Hue1R = SumREDred/TRed;  Hue1G = SumREDgreen/TRed;  Hue1B = SumREDblue/TRed;
            Hue2 = TCyan                                                                                # complementary colour
            Hue2name = "Cyan"
        if Hue1 == TYellow:
            Hue1name = "Yellow"
            Hue1R = SumYELLOWred/TYellow;  Hue1G = SumYELLOWgreen/TYellow;  Hue1B = SumYELLOWblue/TYellow;
            Hue2 = TBlue                                                                                # complementary colour
            Hue2name = "Blue"
        if Hue1 == TGreen:
            Hue1name = "Green"
            Hue1R = SumGREENred/TGreen;  Hue1G = SumGREENgreen/TGreen;  Hue1B = SumGREENblue/TGreen;
            Hue2 = TMagenta                                                                             # complementary colour
            Hue2name = "Magenta"
        if Hue1 == TCyan:
            Hue1name = "Cyan"
            Hue1R = SumCYANred/TCyan;  Hue1G = SumCYANgreen/TCyan;  Hue1B = SumCYANblue/TCyan;
            Hue2 = TRed                                                                                 # complementary colour
            Hue2name = "Red"
        if Hue1 == TBlue:
            Hue1name = "Blue"
            Hue1R = SumBLUEred/TBlue;  Hue1G = SumBLUEgreen/TBlue;  Hue1B = SumBLUEblue/TBlue;
            Hue2 = TYellow                                                                              # complementary colour
            Hue2name = "Yellow"
        if Hue1 == TMagenta:
            Hue1name = "Magenta"
            Hue1R = SumMAGENTAred/TMagenta;  Hue1G = SumMAGENTAgreen/TMagenta;  Hue1B = SumMAGENTAblue/TMagenta;
            Hue2 = TGreen                                                                               # complementary colour
            Hue2name = "Green"
        Hue2R = 255-Hue1R; Hue2G = 255-Hue1G;   Hue2B = 255-Hue1B;                                  # complementary colour components
        
        LumaHue1 = (0.299*Hue1R + 0.587*Hue1G + 0.114*Hue1B) + 1   
        LumaHue2 = (0.299*Hue2R + 0.587*Hue2G + 0.114*Hue2B) + 1   
        #info_msg ("LumaHue 1,2 = "+str(LumaHue1) + " , " + str(LumaHue2))
        
        AvgLuma = TLuma/TPixels
        
        # Iterate again over the pixels of each tile.
        # - Replace RGB values with one of the 2 hues based on luma
  
        for i in range(tn):
            for j in range(tm):
                # Update the progress bar.
                gimp.progress_update(float(i*tm + j) / float(tn*tm))
        
                # Get the tiles.
                srcTile = inDrawable.get_tile(False, j, i)
                dstTile = newLayer.get_tile(False, j, i)
        
                for x in range(srcTile.ewidth):
                    for y in range(srcTile.eheight):
                        # Get the pixel and calculate his gray value.
                        pixel = srcTile[x,y]
                        red = ord(pixel[0])
                        green = ord(pixel[1])
                        blue = ord(pixel[2])
                        gray = (ord(pixel[0]) + ord(pixel[1]) + ord(pixel[2]))/3
                        
                        newred = red
                        newgreen = green
                        newblue = blue
                        
                        Luma = max (1, (0.299*red + 0.587*green + 0.114*blue))  # avoid zero
                        # Luminance (perceived option 1): (0.299*R + 0.587*G + 0.114*B)
                        #if Luma<=AvgLuma:   # darker half
                        if Luma<=127:   # darker half    
                            if LumaHue1 < LumaHue2: # apply Hue1 which is darker
                                newred = min(255,int(Hue1R*Luma/LumaHue1));
                                newgreen = min(255,int(Hue1G*Luma/LumaHue1));
                                newblue = min(255,int(Hue1B*Luma/LumaHue1));
                            else: # apply Hue2 which is darker
                                newred = min(255,int(Hue2R*Luma/LumaHue2));
                                newgreen = min(255,int(Hue2G*Luma/LumaHue2));
                                newblue = min(255,int(Hue2B*Luma/LumaHue2));
                        else:  #lighter half
                            if LumaHue1  > LumaHue2: # apply Hue1 which is lighter
                                newred = min(255,int(Hue1R*Luma/LumaHue1));
                                newgreen = min(255,int(Hue1G*Luma/LumaHue1));
                                newblue = min(255,int(Hue1B*Luma/LumaHue1));
                            else: # apply Hue2 which is lighter
                                newred = min(255,int(Hue2R*Luma/LumaHue2));
                                newgreen = min(255,int(Hue2G*Luma/LumaHue2));
                                newblue = min(255,int(Hue2B*Luma/LumaHue2));
                            
                        res = chr(newred) + chr(newgreen) + chr(newblue)
                        
                        # If the image has an alpha channel (or any other channel) copy his values.
                        if(len(pixel) > 3):
                            for k in range(len(pixel)-3):
                                res += pixel[k+3]
                                
                        # Save the value in the result layer.
                        dstTile[x,y] = res
        
        # Update the new layer.
        newLayer.flush()
        newLayer.merge_shadow (True)
        newLayer.update (0, 0, newLayer.width, newLayer.height)
        
        x_blur = newLayer.width/200
        y_blur = newLayer.height/200
        pdb.plug_in_gauss_iir2 (inImage, newLayer, x_blur, y_blur)
        
        # Remove the old layer if you like (or keep it).
        # inImage.remove_layer(inDrawable)
        
        # Change the name of the new layer (two layers can not have the same name).
        newLayer.name = layerName+" applying 2 hues "
        newLayer.mode = SOFTLIGHT_MODE
        newLayer.opacity = 75.0
        pdb.plug_in_vinvert (inImage, newLayer)
        pdb.plug_in_gauss (inImage, newLayer, 8, 8, 0)
        
    except Exception as err:
        erro_msg ("Unexpected error: " + str(err))
    #info_msg ("Hues 1,2 = "+Hue1name + "," + Hue2name)
    #info_msg ("Hue 1 values R,G,B = "+str(Hue1R)+" , "+str(Hue1G)+" ,"+str(Hue1B))
    #info_msg ("SumREDred = "+str(SumREDred))
    #info_msg ("SumREDgreen = "+str(SumREDgreen))
    #info_msg ("SumREDblue = "+str(SumREDblue))
    #info_msg (str(SortedList))
    #info_msg ("last red = "+str(red))
    #if LumaHue1 < LumaHue2:
     #   info_msg (Hue1name + " is lighter")
    #else:
    #    info_msg (Hue1name + " is darker")
    # Close the undo group.
    pdb.gimp_image_undo_group_end(inImage)
    
    # End progress.
    pdb.gimp_progress_end()
    pdb.gimp_context_pop()
    
    return(newLayer)

register(
    "Only2hues",
    "to create e new layer with only 2 hues, the most present and the complementary",
    "This script creates e new layer with only 2 hues, the most present and the complementary.",
    "JFM-DN",
    "Open source (BSD 3-clause license)",
    "2017",
    "Only2hues",
    "RGB, RGB*",
    [
      (PF_IMAGE, "image", "Input image", None),
      (PF_DRAWABLE, "drawable", "Input drawable", None),
    ],
    [
      (PF_DRAWABLE, "outdrawable", "Output drawable", None),
    ],
    Only2hues,
    menu="<Image>/Diego/Ancillary"
    )

main()
